@@ -13,12 +13,14 @@ use alloc::{format, string::{String, ToString}, vec, vec::Vec};
 //use crate::libcore::fs::directory_entry::FileInfo;
 //use crate::libcore::fs::fname;
 
-use crate::libcore::fs::bmapblk::BMAPSIZE;
-use crate::libcore::fs::dev::{Device, DevType};
-use crate::libcore::fs::directory::Directory;
-use crate::libcore::fs::file::{File, SeekFrom};
-use crate::libcore::fs::blkdev::{fmtata, fmtmem, mounted, mntata, mntmem, dismount};
-use crate::libcore::fs::directory_entry::{DirectoryEntry, FileInfo};
+use crate::serprintln;
+pub use crate::libcore::fs::ata::BLKSIZE;
+pub use crate::libcore::fs::bmapblk::BMAPSIZE;
+pub use crate::libcore::fs::dev::{Device, DevType};
+pub use crate::libcore::fs::directory::Directory;
+pub use crate::libcore::fs::file::{File, SeekFrom};
+pub use crate::libcore::fs::blkdev::{fmtata, fmtmem, mounted, mntata, mntmem, dismount};
+pub use crate::libcore::fs::directory_entry::{DirectoryEntry, FileInfo};
 
 
 pub mod ata;
@@ -138,6 +140,20 @@ pub fn canon(path: &str) -> Result<String, ()>
 }
 
 
+// Create a new device
+pub fn dev_new(path: &str, tp: DevType) -> Option<usize>
+{
+	let flags = OpenFlag::CREATE as usize | OpenFlag::DEVICE as usize;
+	if let Some(handle) = crate::libcore::sys::sc::open(path, flags)
+	{
+		let buffer = [tp as u8; 1];
+		return crate::libcore::sys::sc::write(handle, &buffer);
+	}
+
+	None
+}
+
+
 // Open device
 pub fn dev_open(path: &str) -> Option<usize>
 {
@@ -146,6 +162,22 @@ pub fn dev_open(path: &str) -> Option<usize>
 }
 
 
+// Create a new directory
+pub fn directory_new(path: &str) -> Option<usize>
+{
+	let flags = OpenFlag::CREATE as usize | OpenFlag::DIRECTORY as usize;
+	crate::libcore::sys::sc::open(path, flags)
+}
+
+
+// Open a directory
+pub fn directory_open(path: &str) -> Option<usize>
+{
+	let flags = OpenFlag::DIRECTORY as usize;
+	crate::libcore::sys::sc::open(path, flags)
+}
+
+
 // Disk free
 pub fn diskfree() -> usize
 {
@@ -184,6 +216,13 @@ pub fn dname(pname: &str) -> &str
 }
 
 
+// Exists
+pub fn exists(path: &str) -> bool
+{
+	crate::libcore::sys::sc::info(path).is_some()
+}
+
+
 // Open file
 pub fn file_open(path: &str) -> Option<usize>
 {
@@ -213,6 +252,25 @@ pub fn info(pname: &str) -> Option<FileInfo>
 }
 
 
+// Initialize
+pub fn init()
+{
+	use crate::libcore::fs::sblk::SBlk;
+
+	for bus in 0..2
+	{
+		for disk in 0..2
+		{
+			if SBlk::checkata(bus, disk)
+			{
+				serprintln!("[INFO] FOUND LIBFS SBLK IN ATA DRIVE: {}:{}\n", bus, disk);
+				mntata(bus, disk);
+				return;
+			}
+		}
+	}
+}
+
 // New file
 pub fn new_file(path: &str) -> Option<usize>
 {
@@ -221,6 +279,34 @@ pub fn new_file(path: &str) -> Option<usize>
 }
 
 
+// Read
+pub fn read(path: &str, buffer: &mut [u8]) -> Result<usize, ()>
+{
+	if let Some(info) = crate::libcore::sys::sc::info(&path)
+	{
+		let res = if info.isdev()
+		{
+			dev_open(&path)
+		}
+		else
+		{
+			file_open(&path)
+		};
+
+		if let Some(handle) = res
+		{
+			if let Some(bytes) = crate::libcore::sys::sc::read(handle, buffer)
+			{
+				crate::libcore::sys::sc::close(handle);
+				return Ok(bytes);
+			}
+		}
+	}
+
+	Err(())
+}
+
+
 // Read to bytes
 pub fn read_to_bytes(path: &str) -> Result<Vec<u8>, ()>
 {
@@ -258,6 +344,36 @@ pub fn read_to_str(path: &str) -> Result<String, ()>
 }
 
 
+// Reopen
+pub fn reopen(path: &str, handle: usize) -> Result<usize, ()>
+{
+	let res = if let Some(info) = crate::libcore::sys::sc::info(&path)
+	{
+		if info.isdev()
+		{
+			dev_open(&path)
+		}
+		else
+		{
+			file_open(&path)
+		}
+	}
+	else
+	{
+		new_file(&path)
+	};
+
+	if let Some(original) = res
+	{
+		crate::libcore::sys::sc::dup(original, handle);
+		crate::libcore::sys::sc::close(original);
+		return Ok(handle);
+	}
+
+	Err(())
+}
+
+
 // Real path
 pub fn rpath(pname: &str) -> String
 {
