@@ -6,6 +6,18 @@
 // Console definitions/functionality
 pub mod console;
 
+// CPU
+pub mod cpu;
+
+// Global descriptor table (GDT)
+pub mod gdt;
+
+// Interrupt descriptor table (IDT)
+pub mod idt;
+
+// PCI
+pub mod pci;
+
 // System processes
 pub mod proc;
 
